%This script is used to generate and reduce scenarios for the balancing
%market. The script is run every new hour that Mosel is running, and new
%information is obtained for each hour. 

%Starthour is defined as 5 due to the forecasting properties. Thus we can forecast
%from hour 5-38. Number of hours will always be |Hours| - this hour. 
tic;
clear; 
startTime = tic; 

starthour = 5;
eScen = 10; 
bScen = 30; 
buyProb = 1/eScen; 
sellProb = 1/eScen;
totScen = eScen*bScen; 

H = 39 - starthour; 

%Initialize profit vector for stability testing
Aprofit = zeros(H,1); 

%Initialize profit vector for stability testing
Aintraday = zeros(H,1); 

%Initialize gap vector for stability testing
Agap = zeros(H,1);

%Initialize matrices for data analyses
AelbasSellPrice = zeros(H,24); 
AelbasBuyPrice = zeros(H,24);
AbmPrice = zeros(H,24);
AelbasSellVol = zeros(H,24); 
AelbasBuyVol = zeros(H,24); 
AbmVol = zeros(H,24);
AsellAcc = zeros(H*8,totScen*24); 
AbuyAcc = zeros(H*8, totScen*24);
Asell = zeros(H*totScen, 24);
Abuy = zeros(H*totScen, 24);
AbmAll = zeros(H*totScen, 24);
AsellAll = zeros(H*totScen, 24);
AbuyAll = zeros(H*totScen, 24);
AbmVolAll = zeros(H*totScen, 24);
AbidSellAll = zeros(H*totScen, 24);
AbidBuyAll = zeros(H*totScen, 24);
Aspill = zeros(H*totScen, 24);
AexpBidSell = zeros(H,24); 
AexpBidBuy = zeros(H,24); 

count = 0; 

%Random seed
rng('shuffle'); 

%Create elbasscenarios
elbasScenarios(eScen, bScen, 24);

cd 'C:\xpressmp\matlab'
probMatrix = xlsread('Transition.xlsx', 'Total', 'B2:D73'); 
b2 = xlsread('Casedata, BM.xlsx', '01.06', 'H2:H39');
b3 = xlsread('Casedata, BM.xlsx', '01.06', 'I2:I39');
e2 = xlsread('Casedata, BM.xlsx', 'Residuals', 'A1:A8762'); 
e3 = xlsread('Casedata, BM.xlsx', 'Residuals', 'B1:B8762');
up = xlsread('BMVolumes.xlsx', 'Non-zero data', 'A1:A3882'); 
down = xlsread('BMVolumes.xlsx', 'Non-zero data', 'B1:B5133'); 
nScenarios = 3000; 

%Read the profit obtained from spot
spotProf = importdata('SpotResult0106.txt');
spotProfit = spotProf.data; 

%Read the initial generator start status
genS = importdata('genStart0106.txt');
genStart = genS.data; 

%Create initial delivery files
filename = fopen('Generator1.txt', 'wt');
fprintf(filename, 'Gen1: [\n'); 
for i = 1:totScen
    for h = 1:24
        fprintf(filename, '%d\t', genStart(h,1)); 
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

filename = fopen('Generator2.txt', 'wt');
fprintf(filename, 'Gen2: [\n'); 
for i = 1:totScen
    for h = 1:24
        fprintf(filename, '%d\t', genStart(h,2)); 
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

filename = fopen('Generator3.txt', 'wt');
fprintf(filename, 'Gen3: [\n'); 
for i = 1:totScen
    for h = 1:24
        fprintf(filename, '%d\t', genStart(h,3)); 
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

cd 'C:\xpressmp\matlab'

%Generate volumes for each scenario
volumeUpGeneration(up, 24); 
volumeDownGeneration(down, 24);

filename = fopen('ResScen0106.txt', 'wt'); 
fprintf(filename, 'InitialReservoir: \n'); 

%Create initial reservoir file
before = importdata('InitialReservoir0106.txt'); 
beforeData = before.data;
for s = 1:totScen
    for r = 1:2
        fprintf(filename, '%d\t', beforeData(1,r)); 
    end
    fprintf(filename, '\n'); 
end
fclose(filename); 

%Create initial sold Elbas volumes for each production hour and each
%scenario
filename = fopen('ElbasTotBuy0106.txt', 'wt'); 
fprintf(filename, 'ElbasTotalBuy: [\n'); 
for s = 1:totScen
    for h = 1:24
       fprintf(filename, '%f\t', 0);  
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

filename = fopen('ElbasTotSell0106.txt', 'wt'); 
fprintf(filename, 'ElbasTotalSell: [\n'); 
for s = 1:totScen
    for h = 1:24
       fprintf(filename, '%f\t', 0);  
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

filename = fopen('DynamicElbasBuy0106.txt', 'wt'); 
fprintf(filename, 'ElbasBuy: [\n'); 
for s = 1:totScen
    for h = 1:24
       fprintf(filename, '%f\t', 0);  
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

filename = fopen('DynamicElbasSell0106.txt', 'wt'); 
fprintf(filename, 'ElbasSell: [\n'); 
for s = 1:totScen
    for h = 1:24
       fprintf(filename, '%f\t', 0);  
    end
    fprintf(filename, '\n'); 
end
fprintf(filename, ']\n');
fclose(filename); 

%Reduce the volume scenarios
status = 1; 
while status ~= 0
     status = system('scenred cmd2.txt -nogams');
end
status = 1; 
while status ~= 0
     status = system('parse_scenarios2.py');
end
status = 1; 
while status ~= 0
     status = system('scenred cmd3.txt -nogams');
end
status = 1; 
while status ~= 0
     status = system('parse_scenarios3.py');
end

%Rename the file to avoid writing to the same file
fileUp = 'upVolume.txt'; 
fileDown = 'downVolume.txt'; 
movefile ('scenariosParseBMUpVol.txt', fileUp);  
movefile ('scenariosParseBMDownVol.txt', fileDown);  
%Read the new files
upVol = dlmread(fileUp);
downVol = dlmread(fileDown); 
   
%Rename the file to avoid writing to the same file
filePU = 'upProbability.txt'; 
filePD = 'downProbability.txt'; 
movefile ('probabilityBMUpVol.txt', filePU);
movefile ('probabilityBMDownVol.txt', filePD);
%Read the new files
upProb = dlmread(filePU);
downProb = dlmread(filePD);

for i = 1:(H)
    
    cd 'C:\xpressmp\matlab'
    
    %Create dynamic spot profit-file
    filename = fopen('SpotProfit0106.txt', 'wt'); 
    fprintf(filename, 'SpotProfit: [ %d ]', spotProfit(i)); 
    fclose(filename);
    
    s1 = num2str(starthour); 
    s2 = num2str(i - 1); 
    s3 = num2str(i);
    hours = 39 - starthour; 
    
    if (hours-24) > 0
        forecast = 24; 
    else
        forecast = hours; 
    end
    
    %If forecasted hours is less than 24, make the files smaller
    upVolTemp = upVol(1:bScen,(25-forecast+1):25); 
    downVolTemp = downVol(1:bScen, (25-forecast+1):25); 
       
    %Create the complete volume scenarios
    volumeScen(upVolTemp, downVolTemp, eScen, bScen, forecast)
    
    %Create the Elbasscenarios. If forecasted is less than 24 hours, make
    %smaller. 
    sBuy = importdata('ElbasSlopeBuy.txt'); 
    sBuyVar = sBuy.data(1:totScen, (25-forecast):24); 
    sSell = importdata('ElbasSlopeSell.txt');
    sSellVar = sSell.data(1:totScen, (25-forecast):24); 
    iBuy = importdata('ElbasInterceptBuy.txt');
    iBuyVar = iBuy.data(1:totScen, (25-forecast):24); 
    iSell = importdata('ElbasInterceptSell.txt'); 
    iSellVar = iSell.data(1:totScen, (25-forecast):24); 
    
    filename = fopen('Elbas.txt', 'wt');
    
    fprintf(filename, 'ElbasSlopeBuy: [\n');
    for s = 1:totScen
       for h = 1:forecast
          fprintf(filename, '%f\t', sBuyVar(s,h));   
       end
       fprintf(filename, '\n');        
    end
    fprintf(filename, ']\n'); 
    
    fprintf(filename, 'ElbasSlopeSell: [\n');
    for s = 1:totScen
       for h = 1:forecast
          fprintf(filename, '%f\t', sSellVar(s,h));   
       end
       fprintf(filename, '\n');        
    end
    fprintf(filename, ']\n'); 
    
    fprintf(filename, 'ElbasInterceptBuy: [\n');
    for s = 1:totScen
       for h = 1:forecast
          fprintf(filename, '%f\t', iBuyVar(s,h));   
       end
       fprintf(filename, '\n');        
    end
    fprintf(filename, ']\n'); 
    
    fprintf(filename, 'ElbasInterceptSell: [\n');
    for s = 1:totScen
       for h = 1:forecast
          fprintf(filename, '%f\t', iSellVar(s,h));   
       end
       fprintf(filename, '\n');        
    end
    fprintf(filename, ']\n'); 
    
    fclose(filename); 
    
    %Update the spot volumes and prices
    filename = fopen('DynamicSpot0106.txt', 'wt'); 
    
    spotP = importdata('spot0106.txt'); 
    spotPrice = spotP.data(1, (25-forecast):24); 
    spotV = importdata('spotVol0106.txt'); 
    spotVol = spotV.data(1, (25-forecast):24);
    in = importdata('Inflow0106.txt'); 
    inflow = in.data((25-forecast):24, 1:2); 
    if forecast >= 24 
        gen1 = importdata('Generator1.txt'); 
        genStart1 = gen1.data;
        gen2 = importdata('Generator2.txt'); 
        genStart2 = gen2.data;
        gen3 = importdata('Generator3.txt'); 
        genStart3 = gen3.data;
    else
        gen1 = importdata('Generator1.txt'); 
        genStart1 = gen1.data(1:totScen, 2:forecast + 1);
        gen2 = importdata('Generator2.txt'); 
        genStart2 = gen2.data(1:totScen, 2:forecast + 1);
        gen3 = importdata('Generator3.txt'); 
        genStart3 = gen3.data(1:totScen, 2:forecast + 1);
    end
    fprintf(filename, 'SpotPrice: [\t'); 
    for h1 = 1:forecast
       fprintf(filename, '%f\t', spotPrice(1,h1));       
    end
    fprintf(filename, ']\n');
    fprintf(filename, 'SpotVolume: [\t');
    for h2 = 1:forecast
       fprintf(filename, '%f\t', spotVol(1,h2));       
    end
    fprintf(filename, ']\n');
    fprintf(filename, 'Inflow_hj: [\n');
    for h3 = 1:forecast
        for r1 = 1:2
            fprintf(filename, '%f\t', inflow(h3,r1));
        end
        fprintf(filename, '\n'); 
    end
    fprintf(filename, ']\n');
    fprintf(filename, 'GeneratorStart1: [\n');
    for s = 1:totScen
        for h4 = 1:forecast
            if genStart1(s,h4) ~= 1
                fprintf(filename, '%f\t', 0);
            else
                fprintf(filename, '%f\t', 1); 
            end
        end
        fprintf(filename, '\n'); 
    end
    fprintf(filename, ']\n');
    
    fprintf(filename, 'GeneratorStart2: [\n');
    for s = 1:totScen
        for h4 = 1:forecast
            if genStart2(s,h4) ~= 1
                fprintf(filename, '%f\t', 0);
            else
                fprintf(filename, '%f\t', 1); 
            end
        end
        fprintf(filename, '\n'); 
    end
    fprintf(filename, ']\n');
    
     fprintf(filename, 'GeneratorStart3: [\n');
    for s = 1:totScen
        for h4 = 1:forecast
            if genStart3(s,h4) ~= 1
                fprintf(filename, '%f\t', 0);
            else
                fprintf(filename, '%f\t', 1); 
            end
        end
        fprintf(filename, '\n'); 
    end
    fprintf(filename, ']\n');
    fclose(filename);

    %Update the sold Elbasvolumes from earlier bid hours
    elbB = importdata('DynamicElbasBuy0106.txt'); 
    elbasB = elbB.data; 
    elbS = importdata('DynamicElbasSell0106.txt'); 
    elbasS = elbS.data;
    etotbuy = importdata('ElbasTotBuy0106.txt'); 
    elbTotB = etotbuy.data; 
    etotsell = importdata('ElbasTotSell0106.txt'); 
    elbTotS = etotsell.data;   
    newElbB = elbasB + elbTotB; 
    newElbS = elbasS + elbTotS;
          
    if forecast < 24
        
        dynB = strcat('DynamicElbasBuy0106', s2, '.txt'); 
        filename1 = fopen(dynB, 'wt');  
        fprintf(filename1, 'ElbasBuy: [\n');
        for s = 1:totScen
            for h = 1:(forecast + 1)
                fprintf(filename1, '%f\t', elbasB(s,h)); 
            end 
            fprintf(filename1, '\n');
        end
        fprintf(filename1, ']\n'); 
        fclose(filename1); 
    
        dynS = strcat('DynamicElbasSell0106', s2, '.txt'); 
        filename1 = fopen(dynS, 'wt');  
        fprintf(filename1, 'ElbasSell: [\n');
        for s = 1:totScen
            for h = 1:(forecast + 1)
                fprintf(filename1, '%f\t', elbasS(s,h)); 
            end
            fprintf(filename1, '\n');
        end
        fprintf(filename1, ']\n'); 
        fclose(filename1); 
        fileDynB = strcat('ElbasTotBuy0106', s3, '.txt'); 
        filename = fopen('ElbasTotBuy0106.txt', 'wt'); 
        filename1 = fopen(fileDynB, 'wt');  
        fprintf(filename, 'ElbasTotalBuy: [\n');
        fprintf(filename1, 'ElbasTotalBuy: [\n');
        for s = 1:totScen
            for h = 2:(forecast + 1)
                fprintf(filename, '%f\t', newElbB(s,h));
                fprintf(filename1, '%f\t', newElbB(s,h)); 
            end
            fprintf(filename, '\n'); 
            fprintf(filename1, '\n');
        end
        fprintf(filename, ']\n'); 
        fclose(filename); 
        fprintf(filename1, ']\n'); 
        fclose(filename1);
    
        fileDynS = strcat('ElbasTotSell0106', s3, '.txt'); 
        filename = fopen('ElbasTotSell0106.txt', 'wt');
        filename1 = fopen(fileDynS, 'wt');
        fprintf(filename, 'ElbasTotalSell: [\n'); 
        fprintf(filename1, 'ElbasTotalSell: [\n');
        for s = 1:totScen
            for h = 2:(forecast + 1)
                fprintf(filename, '%f\t', newElbS(s,h));
                fprintf(filename1, '%f\t', newElbS(s,h));
            end
            fprintf(filename, '\n'); 
            fprintf(filename1, '\n'); 
        end
        fprintf(filename, ']\n'); 
        fclose(filename);
        fprintf(filename1, ']\n'); 
        fclose(filename1);
        
    else
        dynB = strcat('DynamicElbasBuy0106', s2, '.txt'); 
        filename1 = fopen(dynB, 'wt');  
        fprintf(filename1, 'ElbasBuy: [\n');
        for s = 1:totScen
            for h = 1:forecast
                fprintf(filename1, '%f\t', elbasB(s,h)); 
            end 
            fprintf(filename1, '\n');
        end
        fprintf(filename1, ']\n'); 
        fclose(filename1); 
    
        dynS = strcat('DynamicElbasSell0106', s2, '.txt'); 
        filename1 = fopen(dynS, 'wt');  
        fprintf(filename1, 'ElbasSell: [\n');
        for s = 1:totScen
            for h = 1:forecast
                fprintf(filename1, '%f\t', elbasS(s,h)); 
            end
            fprintf(filename1, '\n');
        end
        fprintf(filename1, ']\n'); 
        fclose(filename1); 
        
        fileDynB = strcat('ElbasTotBuy0106', s3, '.txt'); 
        filename = fopen('ElbasTotBuy0106.txt', 'wt'); 
        filename1 = fopen(fileDynB, 'wt');  
        fprintf(filename, 'ElbasTotalBuy: [\n');
        fprintf(filename1, 'ElbasTotalBuy: [\n');
        for s = 1:totScen
            for h = 1:(forecast)
                fprintf(filename, '%f\t', newElbB(s,h));
                fprintf(filename1, '%f\t', newElbB(s,h)); 
            end
            fprintf(filename, '\n'); 
            fprintf(filename1, '\n');
        end
        fprintf(filename, ']\n'); 
        fclose(filename); 
        fprintf(filename1, ']\n'); 
        fclose(filename1);
    
        fileDynS = strcat('ElbasTotSell0106', s3, '.txt'); 
        filename = fopen('ElbasTotSell0106.txt', 'wt');
        filename1 = fopen(fileDynS, 'wt');
        fprintf(filename, 'ElbasTotalSell: [\n'); 
        fprintf(filename1, 'ElbasTotalSell: [\n');
        for s = 1:totScen
            for h = 1:forecast
                fprintf(filename, '%f\t', newElbS(s,h));
                fprintf(filename1, '%f\t', newElbS(s,h));
            end
            fprintf(filename, '\n'); 
            fprintf(filename1, '\n'); 
        end
        fprintf(filename, ']\n'); 
        fclose(filename);
        fprintf(filename1, ']\n'); 
        fclose(filename1);
    end
    
    %Update the initial water volumes, if remaining time is less than 24
    %hours
    filename = fopen('Reservoir0106.txt', 'wt'); 
    fprintf(filename, 'InitialWaterVolume: [\n'); 
    
    if forecast >= 24
       before = importdata('ResScen0106.txt'); 
       beforeData = before.data;
       for s = 1:totScen
           for r = 1:2
               fprintf(filename, '%f\t', beforeData(s,r));
           end
           fprintf(filename, '\n'); 
       end
    else
       after = importdata('DynamicReservoir0106.txt'); 
       afterData = after.data;
       
       for s = 1:totScen
           for r = 1:2
               fprintf(filename, '%f\t', afterData(s,r));
           end
           fprintf(filename, '\n'); 
       end

    end
    
    fprintf(filename, ']\n'); 
    fclose(filename);   
    
    %Create forecast of the BM-premiums. Starthour is this hour. 
    forecastBMMaster(probMatrix, nScenarios, hours, b2, b3, e2, e3, starthour, forecast); 
    
    starthour = starthour + 1;
    
    %Run SCENRED and reduce scenarios
    %status = 1; 
    %while status ~= 0
    system('scenred cmd.txt -nogams');
    %end
        status = 1; 
    %while status ~= 0
    system('parse_scenarios.py');
    %end
    %Rename the file to avoid writing to the same file
    file1 = strcat('scenariosParseBM', s1, '.txt');
    movefile ('scenariosParseBM.txt', file1);  
    %Read the new files
    matrix = dlmread(file1); 
   
    %Rename the file to avoid writing to the same file
    file2 = strcat('probabilityBM', s1, '.txt'); 
    movefile ('probabilityBM.txt', file2);
    %Read the new files
    bmProb = dlmread(file2);

    %Merge the scenarios and create a textfile for the BM-premiums and the
    %Elbas data. Compute individual scenario probabilities. 
    
    %BM-scenarios
    premium = premScen(matrix, i, forecast, eScen, bScen);
    
    %Probabilities
    scenprob(bmProb, upProb, downProb, buyProb, sellProb, eScen, bScen); 

    %Add Matlab/Xpress-path
    addpath ('C:\xpressmp\matlab'); 
 
    moselexec ('0106.mos'); 
    
    %Create profit vector
    prof = importdata('Profit0106.txt'); 
    
    Aprofit(i) = prof; 
    
    %Create intraday vector
    intra = importdata('Intra0106.txt'); 
    
    Aintraday(i) = intra; 
    
    %Create gap vector
    gap = importdata('Gap.txt'); 
    
    Agap(i) = gap; 
    
    %Create matrices for data analyses 
    esPrice = importdata('ElbasPriceSell.txt');
    AelbasSellPrice(i,(25-forecast):24) = esPrice; 
    
    ebPrice = importdata('ElbasPriceBuy.txt');
    AelbasBuyPrice(i,(25-forecast):24) = ebPrice; 
    
    bmP = importdata('BMprice.txt');
    AbmPrice(i,(25-forecast):24) = bmP; 
    
    esV = importdata('ElbasVolumeSell.txt'); 
    AelbasSellVol(i,(25-forecast):24) = esV; 
    
    ebV = importdata('ElbasVolumeBuy.txt'); 
    AelbasBuyVol(i,(25-forecast):24) = ebV; 
    
    bmV = importdata('BMvolume.txt');
    AbmVol(i,(25-forecast):24) = bmV;
    
    sAcc = importdata('ElbasAccSell.txt'); 
    [~,b] = size(sAcc);
    AsellAcc((count*8)+1:(count*8)+8,1:b) = sAcc;
    
    bAcc = importdata('ElbasAccBuy.txt'); 
    [~,b] = size(bAcc);
    AbuyAcc((count*8)+1:(count*8)+8,1:b) = bAcc;
    
    eS = importdata('Sell.txt'); 
    Asell((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = eS; 
    
    eB = importdata('Buy.txt'); 
    Abuy((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = eB; 
    
    bmAll = importdata('BMall.txt');
    AbmAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = bmAll;     
    
    sAll = importdata('SellAll.txt');
    AsellAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = sAll; 
    
    bAll = importdata('BuyAll.txt');
    AbuyAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = bAll; 
   
    bVAll = importdata('BMvolAll.txt');
    AbmVolAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = bVAll; 
    
    bidSAll = importdata('BidVolSellAll.txt');
    AbidSellAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = bidSAll;
    
    bidBAll = importdata('BidVolBuyAll.txt');
    AbidBuyAll((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = bidBAll; 
    
    sp = importdata('Spill.txt');
    Aspill((totScen*count)+1:(count*totScen)+totScen,(25-forecast):24) = sp; 
    
    xbS = importdata('ExpBidSell.txt');
    AexpBidSell(i,(25-forecast):24) = xbS;

    xbB = importdata('ExpBidBuy.txt');
    AexpBidBuy(i,(25-forecast):24) = xbB;
   
    count = count + 1; 
end

toc 
stopTime = toc; 

timeUsed = toc - tic;